/***************************************************************************
                          binarytree.h  -  description
                             -------------------
    begin                : Wed Jul 13 2005
    copyright            : (C) 2005 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  Description:
Objects of BinaryTree are nodes of a binary tree with two
successors. Trees can be constructed by combining
BinaryTree-objects. Here the class is used to describe
coalescent trees. An evolutionary process can be performed
along a tree to generate synthetic data.
*/

#ifndef BINARYTREE_H
#define BINARYTREE_H

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>
#include "randomnumbergenerator.h"

/**
  *@author Michael Rosskopf
  */

class BinaryTree {

  public: 

  // Constructors and destructors
  BinaryTree();
  ~BinaryTree();

  // getLeafNumber: Returns the leaf number. Returns '0' for an inner node
  long getLeafNumber();

  // getValue: Returns the stored value (it is used here to describe the gene expression)
  double getValue();

  // getLeftChild: Returns the left child. Returns '0' if left child is empty.
  BinaryTree* getLeftChild();

  // getRightchild: Returns the right child. Returns '0' if right child is empty.
  BinaryTree* getRightchild();

  // getEdgeLengthToLeftChild: Returns length of the edge to the left child. Returns '0' if left child is empty.
  double getEdgeLengthToLeftChild();

  // getEdgeLengthToRightChild: Returns length of the edge to the right child. Returns '0' if right child is empty.
  double getEdgeLengthToRightChild();

  // getNumberOfLeaves: Returns the number of leaves of the tree.
  // If the tree is a leaf, 1 is returned.
  long getNumberOfLeaves();

  // setValue: Set the value of the tree
  void setValue(double);

  // toString: Output of the tree in Newick-Format
  void toString();

  // init: Initialises the tree recursively from a string in Newick-Format
  void init(char*);

  // evolution: Simulates evolution along the tree.
  // Simulated gene expression values are stores in the leaves.
  void evolution(double,double,double,double[],RandomNumberGenerator*);

  private:

  long leafNumber; // Each leaf has an identifier. If the tree is a leaf (no children), leafNumber is the identifier.
  double value; // Each tree can store a value (it is used here to describe the gene expression).
  BinaryTree* leftChild; // Pointer to the left child which is a BinaryTree as well
  BinaryTree* rightChild; // Pointer to the right child which is a BinaryTree as well
  double edgeLengthToLeftChild; // Length of the edge to the left child
  double edgeLengthToRightChild; // Length of the edge to the right child

  // A tree is read from a string in the Newick-Format.
  // The following char-strings are used for the recursive reconstruction of a tree from the string
  char* edgeLengthToLeftChildString; // Length of the edge to the left child (string format)
  char* treeStringLeftChild; // String for the tree with the left child as the root
  char* edgeLengthToRightChildString; // Length of the edge to the right child (string format)
  char* treeStringRightChild; // String for the tree with the right child as the root
};

#endif
