/***************************************************************************
                          controller.h  -  description
                             -------------------
    begin                : Mon Aug 7 2006
    copyright            : (C) 2006 by 
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  Description:
A Controller-object implements all use cases of the package and is initialized
directly from the Emogeetools-object.
*/

#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "programparameters.h"
#include "data.h"
#include "mathfunctions.h"
#include "randomnumbergenerator.h"
#include "thetaallelesestimator.h"
#include "binarytree.h"
          
/**
  *@author 
  */

class Controller {

  public: 

  // Constructors and destructors
  Controller();
  ~Controller();

  // Implementation of the Bayesian mutation detection method
  void performBayesianMutationDetection();

  // Implementation of the Tajima-type test
  void performTajimaTypeTest();

  // calcConfidenceLimits: Calculates the confidence limits of the
  //                        Delta-value distribution
  void calcConfidenceLimits(double,double,double,double*,double*);

  // calcMeanPairwiseDistance: Calculates the mean pairwise distance of a 
  //                            number of sequences generated by ms (by R. Hudson)
  double calcMeanPairwiseDistance(char*,long,long);

  private:

  ProgramParameters* programParameters; // Program parameters which specifies what to do
  Data* data; // data contains a data set loaded for analysis
  MathFunctions* ms; // ms is used for some math functions
  RandomNumberGenerator* rnd; // The Random number generator is used to simulate the M&E normal model in order to generate data

  // Some necessary variables  
  double average;
  double averageDeviation;
  double standardDeviation;
  double variance;
  double skewness;
  double kurtosis;
  long i,j,k,l; // Counter    
};

#endif
