/***************************************************************************
                          data.h  -  description
                             -------------------
    begin                : Thu Feb 17 2005
    copyright            : (C) 2005 by Michael Rosskopf
    email                : rosskopf@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  Description:
An Data-object stores a complete data set generated by simulation or loaded from
a file. Methods for analysis are included.
*/

#ifndef DATA_H
#define DATA_H

#include <iostream.h>
#include <fstream.h>
#include "mathfunctions.h"
#include "randomnumbergenerator.h"
#include "programparameters.h"

/**
  *@author Michael Rosskopf
  */

class Data {

  public: 

  // Constructors and destructors
  Data();
  ~Data();

  // Initialisation
  void init(long,long);
  void init(long,long,long);

  // get- and set-methods
  long getNumberOfSamples();
  long getNumberOfIndividuals();
  long getNumberOfGenes();
  long getLabel(long);
  long getSampleSize(long);
  long getSampleStart(long);
  char* getProbeSetID(long);
  double getGene(long,long);
  void setLabel(long,long);
  void setSampleSize(long,long);
  void setSampleStart(long,long);
  void setGene(long,long,double);

  // toString
  void toString();

  // dataCheck: Checks whether sample labels are in the data set
  bool dataCheck(ProgramParameters*);

  // calcGeneVariance: Calculates the variance of a gene in a specific sample of individuals
  double calcGeneVariance(long,long);

  // calcNumberOfAlleles: Calculates the number of alleles
  double calcNumberOfAlleles(long,long,double,double);

  // Methods for loading and saving data
  void checkDataSize(char*);
  void load(char*);
  void save(char*);

  private:
  long numberOfSamples;     // Number of samples. One sample consists of individuals of one group.
  long numberOfIndividuals; // Overall number of individuals of all samples.
  long numberOfGenes;       // Number of genes
  long* labels;             // Each individual has a label which describes the sample it belongs to.
  long* sampleSize;         // Number of individuals in the different samples
  long* sampleStart;        // Index of the first individual of a sample
  double* genes;            // Stores the gene expression values
  char** probeSetID;        // Stores the probe set IDs of the genes
};

#endif
