/***************************************************************************
                          mathfunctions.cpp  -  description
 ***************************************************************************/

#include "mathfunctions.h"

// Constructors and destructors
MathFunctions::MathFunctions(){
}
MathFunctions::~MathFunctions(){
}

// Operations

// laGamma: Calculates the natural log of the gamma function
double MathFunctions::lnGamma(double value){
  double k;
  double l;
  double tmp;
  double buffer;
  static double cof[6] = {76.18009172947146,
                          -86.50532032941677,
                          24.01409824083091,
                          -1.231739572450155,
                          0.1208650973866179e-2,
                          -0.5395239384953e-5};
  int i;
  l = k = value;
  tmp = k + 5.5;
  tmp = tmp - (k + 0.5) * log(tmp);
  buffer = 1.000000000190015;
  for(i = 0; i <= 5; i++){
    buffer = buffer + cof[i] / ++l;
  }
  return -tmp + log(2.5066282746310005 * buffer / k);
}

// calcMedianRearrange: Calculates the median of the empirical distribution data[] with n elements
//                      (The array will be rearranged!)
void MathFunctions::calcMedianRearrange(double data[], long n, double *median){
  if((n % 2) == 1){
    *median = getISmallestValue(data,(n + 1) / 2,n);
  } else {
    *median = (getISmallestValue(data,n / 2,n) + getISmallestValue(data,(n / 2) + 1,n)) / 2.0;
  }
}

// calcMode: Returns the mode of a distribution which is binned in buckets
// The smallest and the greatest element have to be calculated before.
double MathFunctions::calcMode(long buckets[], long numberOfBuckets, double
smallest, double greatest){
  double mode;
  long numberOfValuesInFullestBucket = 0;
  double range = greatest - smallest;
  if(range < 0){
    range = range * (-1);
  }
  for(long i = 0; i < numberOfBuckets; i++){
    if(numberOfValuesInFullestBucket < buckets[i]){
      numberOfValuesInFullestBucket = buckets[i];
      mode = i;
    }
  }
  mode = (double(mode) / double(numberOfBuckets - 1) * range) + smallest;
  return mode;
}

// calcMoments: Calculates the moments of a distribution data[] of length n
void MathFunctions::calcMoments(double data[], long n, double *average, double *averageDeviation,
double *standardDeviation, double *variance, double *skewness, double *kurtosis){
  long j;
  double ep = 0.0;
  double s;
  double p;
  if(n <= 1){
    *average = data[0];
    *averageDeviation = (*variance) = (*skewness) = (*kurtosis) = 0.0;
  } else {
    s = 0.0;
    for(j = 0; j < n; j++){
      s = s + data[j];
    }
    *average = s/n;
    *averageDeviation = (*variance) = (*skewness) = (*kurtosis) = 0.0;
    for(j = 0; j < n; j++){
      *averageDeviation = *averageDeviation + fabs(s = data[j] - (*average));
      ep += s;
      *variance = *variance + (p = s * s);
      *skewness = *skewness + (p *= s);
      *kurtosis = *kurtosis + (p *= s);
    }
    *averageDeviation = *averageDeviation / n;
    *variance = (*variance - ep * ep / n) / (n - 1);
    *standardDeviation = sqrt(*variance);
    if(*variance){
      *skewness = *skewness / (n * (*variance) * (*standardDeviation));
      *kurtosis = *kurtosis / (n * (*variance) * (*variance));
    }
  }
}

// sort: Sort the array[low...high] with a quicksort algorithm
void MathFunctions::sort(double values[], int low, int high){
  // Lokal varianceiables
  int i = low;                            // Lower limit
  int j = high;                           // Upper limit
  double buffer;                           // Buffer
  double x = values[(low + high) / 2];     // Value of pivotelement
  // Sort routine
  do{
    while(values[i] < x){
      i++;
    }
    while(values[j] > x){
      j--;
    }
    if(i <= j){
      buffer = values[i];
      values[i] = values[j];
      values[j] = buffer;
      i++;
      j--;
    }
  }while(i <= j);
  // Rekursive calls
  if(low < j){
    sort(values,low,j);
  }
  if(i < high){
    sort(values,i,high);
  }
}

// getISmallestValue: Returns the i-th smallest value in the array data[] of length n. Input will be rearranged.
//                    (used by calcMedianRearrange).
double MathFunctions::getISmallestValue(double data[],long i,long n){

  // Necessary, because n is used as an upper limit
  n--;

  long v;
  long w;
  long x;
  long y;
  long middle;
  double buffer;
  double tmp;

  y = 1;
  w = n;
  while(true){
    if(w <= y + 1){
      if(w == y + 1 && data[w] < data[y]){
        tmp = data[y];
        data[y] = data[w];
        data[w] = tmp;
      }
      return data[i];
    } else {
      middle = (y + w) >> 1;
      tmp = data[middle];
      data[middle] = data[y + 1];
      data[y + 1] = tmp;
      if(data[y] > data[w]){
        tmp = data[y];
        data[y] = data[w];
        data[w] = tmp;
      }
      if(data[y + 1] > data[w]){
        tmp = data[y + 1];
        data[y + 1] = data[w];
        data[w] = tmp;
      }
      if(data[y] > data[y + 1]){
        tmp = data[y];
        data[y] = data[y + 1];
        data[y + 1] = tmp;
      }
      v = y + 1;
      x = w;
      buffer = data[y + 1];
      while(true){
        do{
          v++;
        } while(data[v] < buffer);
        do{
          x--;
        } while(data[x] > buffer);
        if(x < v) break;
        tmp = data[v];
        data[v] = data[x];
        data[x] = tmp;
      }
      data[y + 1] = data[x];
      data[x] = buffer;
      if(x >= i) w = x - 1;
      if(x <= i) y = v;
    }
  }
}
