/***************************************************************************
                          mathfunctions.h  -  description
 ***************************************************************************/

/*  Description:
MathFunctions implements some important math functions, for example,
for moment estimation and sorting.
*/

#ifndef MATHFUNCTIONS_H
#define MATHFUNCTIONS_H

#include <math.h>
#include <iostream.h>

class MathFunctions {

public:

  // Constructors and destructors
  MathFunctions();
  ~MathFunctions();

  // lnGamma: Calculates the natural log of the gamma function.
  double lnGamma(double);

  // calcMedianRearrange: Calculates the median of the empirical distribution data[] with n elements
  void calcMedianRearrange(double[],long,double*);

  // calcMode: Returns the mode of a distribution which is binned in buckets
  double calcMode(long[],long,double,double);

  // calcMoments: Calculates the moments of a distribution data[] of length n
  void calcMoments(double[],long,double*,double*,double*,double*,double*,double*);

  // sort: Sort the array[low...high] with a quicksort algorithm
  void sort(double[],int,int);

  // getISmallestValue: Returns the i-th smallest value in the array data[] of length n. Input will be rearranged.
  //                    (used by calcMedianRearrange).
  double getISmallestValue(double[],long,long);

};

#endif
