/***************************************************************************
 *   Copyright (C) 2007 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "programparameters.h"

// Constructors and destructors
ProgramParameters::ProgramParameters(){

  // Read the program parameters from the file config.txt
  char charbuffer;
  char stringbuffer[4096];
  ifstream file("config.txt");

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  tool = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  mode = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  dataTableFilename = new char[sizeof(stringbuffer)];
  strcpy(dataTableFilename,stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sample1 = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sample2 = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sigmaM = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sigmaE = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  d = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  numberOfSimulations = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sample1Size = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  sample2Size = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  maxMut = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  fixedMut = atof(stringbuffer);

  file.get(stringbuffer,4096,'=');
  do{file.get(charbuffer); charbuffer = file.peek();} while(charbuffer == ' '); 
  file.get(stringbuffer,4096,';');
  confidenceInterval = atof(stringbuffer);

  file.close();

  // Formated screen output of the program parameters
  if(tool == 1) cout << "\nTool: Bayesian mutation detection" << endl;
  if(tool == 2) cout << "\nTool: Tajima-type test" << endl;

  if(mode == 1) cout << "\nMode: Create null-distribution" << endl;
  if(mode == 2) cout << "\nMode: Perform a full analysis on real data" << endl;

  cout << "\nData-File: " << dataTableFilename << endl;
  cout << "Sample 1:  " << sample1 << endl;
  if(tool != 2) cout << "Sample 2:  " << sample2 << endl;
  if(tool == 2) cout << "Sample 2:  -" << endl;

  cout << "\nParameters for the null-distribution" << endl;
  cout << "sigma_m: " << sigmaM << endl;
  cout << "sigma_e: " << sigmaE << endl;
  if(tool == 1) cout << "d:       " << d << endl;
  if(tool == 2) cout << "theta:   " << d << endl;

  cout << "\nNumber of genes: " << numberOfSimulations << endl;

  cout << "\nGroup sizes for the null-distribution" << endl;
  if(mode == 1){
    cout << "Size of sample 1: " << sample1Size << endl;
    if(tool != 2) cout << "Size of sample 2: " << sample2Size << endl;
    if(tool == 2) cout << "Size of sample 2: -" << endl;
  } else {
    cout << "Size of sample 1: Equal to data" << endl;
    if(tool != 2) cout << "Size of sample 2: Equal to the size in the data set" << endl;
    if(tool == 2) cout << "Size of sample 2: -" << endl;
  }

  cout << "\nAdditional parameters for the mutation test" << endl;
  if(tool == 1){
    cout << "Maximal number of mutation events:            " << maxMut  << endl;
    cout << "Fixed number of mutation events (simulation): " << fixedMut  << endl;
    cout << "Confidence interval:                          -" << endl;
  }
  if(tool == 2){
    cout << "Maximal number of mutation events:            -" << endl;;
    cout << "Fixed number of mutation events (simulation): -" << endl;;
    cout << "Confidence interval:                          " << confidenceInterval * 100.0 << "%" << endl;
  }
  cout << "\n----------------------------------------------------------------" << endl;
}
ProgramParameters::~ProgramParameters(){
}

// get-methods
int ProgramParameters::getTool(){
  return tool;
}
int ProgramParameters::getMode(){
  return mode;
}
char* ProgramParameters::getDataTableFilename(){
  return dataTableFilename;
}
int ProgramParameters::getSample1(){
  return sample1;
}
int ProgramParameters::getSample2(){
  return sample2;
}
double ProgramParameters::getSigmaM(){
  return sigmaM;
}
double ProgramParameters::getSigmaE(){
  return sigmaE;
}
double ProgramParameters::getD(){
  return d;
}
long ProgramParameters::getNumberOfSimulations(){
  return numberOfSimulations;
}
int ProgramParameters::getSample1Size(){
  return sample1Size;
}
int ProgramParameters::getSample2Size(){
  return sample2Size;
}
int ProgramParameters::getMaxMut(){
  return maxMut;
}
int ProgramParameters::getFixedMut(){
  return fixedMut;
}
double ProgramParameters::getConfidenceInterval(){
  return confidenceInterval;
}

// toString
void ProgramParameters::toString(){
  cout << "Program parameters:" << endl;
  cout << "tool = " << tool << endl;
  cout << "mode = " << mode << endl;
  cout << "dataTableFilename = " << dataTableFilename << endl;
  cout << "sample1 = " << sample1 << endl;
  cout << "sample2 = " << sample2 << endl;
  cout << "sigmaM = " << sigmaM << endl;
  cout << "sigmaE = " << sigmaE << endl;
  cout << "d = " << d << endl;
  cout << "numberOfSimulations = " << numberOfSimulations << endl;
  cout << "sample1Size = " << sample1Size << endl;
  cout << "sample2Size = " << sample2Size << endl;
  cout << "maxMut = " << maxMut << endl;
  cout << "fixedMut = " << fixedMut << endl;
  cout << "confidenceInterval = " << confidenceInterval << endl;
}

// checkProgramParameters: Returns true if program parameters are valid
bool ProgramParameters::checkProgramParameters(){
 
  bool valid = true;

  if(tool != 1 && tool != 2){cout << "\nError: tool " << tool << " not available."; valid = false;}
  if(mode != 1 && mode != 2){cout << "\nError: mode " << mode << " not available."; valid = false;}
  if(sample1 < 1){cout << "\nError: sample1 must be larger than 0." << endl; valid = false;}
  if(sample2 < 1 && tool == 1){cout << "\nError: sample2 must be larger than 0." << endl; valid = false;}
  if(sigmaM <= 0.0){cout << "\nError: sigmaM must be larger than 0." << endl; valid = false;}
  if(sigmaE < 0.0){cout << "\nError: sigmaE must at least 0." << endl; valid = false;}
  if(numberOfSimulations < 1){cout << "\nError: numberOfSimulations must be larger than 0." << endl; valid = false;}
  if(sample1Size < 1){cout << "\nError: sample1Size must be larger than 0." << endl; valid = false;}
  if(sample1Size < 1){cout << "\nError: sample2Size must be larger than 0." << endl; valid = false;}
  if(maxMut < 1){cout << "\nError: maxMut must be larger than 0." << endl; valid = false;}
  if(fixedMut < 0){cout << "\nError: fixedMut must be at least 0." << endl; valid = false;}
  if(confidenceInterval <= 0.0 || confidenceInterval > 1.0){cout << "\nError: confidenceInterval must be between 0 and 1." << endl; valid = false;}
  if(tool == 2 && mode == 1 && sample1Size < 2){
    cout << "\nError: sample1Size must be larger than 1 for tool == 2 and mode == 1." << endl; valid = false;
  }

  return valid;
}
