/***************************************************************************
 *   Copyright (C) 2007 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*  Description:
A ProgramParameter-object reads the program parameters from the config.txt-file
which is used to specify the program execution.
*/

#ifndef PROGRAMPARAMETERS_H
#define PROGRAMPARAMETERS_H

#include <iostream.h>
#include <fstream.h>

/**
	@author Michael Rosskopf <rosskopf@cs.uni-duesseldorf.de>
*/

class ProgramParameters {

  public: 

  // Constructors and destructors
  ProgramParameters();
  ~ProgramParameters();

  // get-methods
  int getTool();
  int getMode();
  char* getDataTableFilename();
  int getSample1();
  int getSample2();
  double getSigmaM();
  double getSigmaE();
  double getD();
  long getNumberOfSimulations();
  int getSample1Size();
  int getSample2Size();
  int getMaxMut();
  int getFixedMut();
  double getConfidenceInterval();

  // toString
  void toString();

  // checkProgramParameters: Returns true if program parameters are valid
  bool checkProgramParameters();

  private:

  int tool; // 1 -> Bayesian mutation detection
            // 2 -> Tajima-type test
  int mode; // 1 -> Create null-distribution
            // 2 -> Perform a full analysis on real data
  char* dataTableFilename; // Char-String for the data table
  int sample1; // Label for the first sample
  int sample2; // Label for the second sample
  double sigmaM; // M-normal model parameter for the standard deviation of the MED
  double sigmaE; // M-normal model parameter for the standard deviation of the N-MED
  double d; // M-normal model parameter for the mean number of mutations
  long numberOfSimulations; // Size of the synthetic gene expression difference distributions
  int sample1Size; // Size of the first sample for generated synthetic data
  int sample2Size; // Size of the second sample for generated synthetic data
  int maxMut; // Maximal number of mutations detected by the Bayesian mutation detection method
  int fixedMut; // Fixed number of mutations per gene if synthetic data is generated (only used by the Bayesian mutation detection method)
  double confidenceInterval; // Confidence limits of the null distribution (only used by the Tajima-type test)
};

#endif
