/***************************************************************************
                          randomnumbergenerator.cpp  -  description
 ***************************************************************************/

#include "randomnumbergenerator.h"

// Constructors and destructors
RandomNumberGenerator::RandomNumberGenerator(){
  seed = -1;
}
RandomNumberGenerator::~RandomNumberGenerator(){
}

// Attributes
// getSeed: Returns the current seed-value
long RandomNumberGenerator::getSeed(){
  return seed;
}
// init: Initialisation of the random number generator with the current system time
void RandomNumberGenerator::init(){
  seed = -(time(0));
}
// initWithModifier: Initialisation of the random number generator with the current system time plus an offset
// (Important for parallel jobs)
void RandomNumberGenerator::initWithModifier(double rndModifier){
  seed = -(time(0)) + rndModifier;
}
// init: Initialisation of the random counter with an explicit value (it must be NEGATIVE!)
void RandomNumberGenerator::init(long _seed){
  seed = _seed;
}

// Probability distributions

// getUniformDistributedNumber: Draws a random number from a uniform probability distribution.
// Parameter: low = lower interval limit, high = upper interval limit
double RandomNumberGenerator::getUniformDistributedNumber(double low, double high){
  double range = high - low;
  if(range < 0) range = range * (-1);
  return (rnd(&seed) * range) + low;
}
// getExtremeValueDistributedNumber: Draws a random number from an extreme value probability distribution.
// Parameter: a, b
double RandomNumberGenerator::getExtremeValueDistributedNumber(double a,double b){
  double dum;
  do{
    dum = rnd(&seed);
  } while (dum == 0.0);
  return a - b * log(-log(dum));
}
// getNormalDistributedNumber: Draws a random number from a normal probability distribution.
// Parameter: mu = mean, sigma = standard deviation
double RandomNumberGenerator::getNormalDistributedNumber(double mu, double sigma){
  rnd(&seed);
  static int m = 0;
  static double n;
  double func;
  double buffer;
  double x1;
  double x2;
  if(seed < 0){
    m = 0;
  }
  if(m == 0){
    do{
      x1 = 2.0 * rnd(&seed) - 1.0;
      x2 = 2.0 * rnd(&seed) - 1.0;
      buffer = x1 * x1 + x2 * x2;
    } while (buffer >= 1.0 || buffer == 0.0);
    func = sqrt(-2.0 * log(buffer) / buffer);
    n = x1 * func;
    m = 1;
    return ((x2 * func) * sigma) + mu;
  } else {
    m = 0;
    return (n * sigma) + mu;
  }
}
// getPoissonDistributedNumber: Draws a random number from a poisson probability distribution.
// Parameter: parameter
double RandomNumberGenerator::getPoissonDistributedNumber(double parameter){
  MathFunctions mathFunctions = MathFunctions();
  rnd(&seed);
  static double k;
  static double logparameter;
  static double l;
  static double x = (-1.0);
  double y;
  double n;
  double m;
  if(parameter < 12.0){
    if(parameter != x){
      x = parameter;
      l = exp(-parameter);
    }
    y = -1;
    n = 1.0;
    do{
      ++y;
      n *= rnd(&seed);
    } while(n > l);
  } else {
    if(parameter != x){
      x = parameter;
      k = sqrt(2.0 * parameter);
      logparameter = log(parameter);
      l = parameter * logparameter - mathFunctions.lnGamma(parameter + 1.0);
    }
    do{
      do{
        m = tan(PI*rnd(&seed));
        y = k * m + parameter;
      } while(y < 0.0);
      y = floor(y);
      n = 0.9 * (1.0 + m *m) * exp(y * logparameter - mathFunctions.lnGamma(y + 1.0) - l);
    } while(rnd(&seed) > n);
  }
  mathFunctions.MathFunctions::~MathFunctions();
  return y;
}

// rnd: Implements the random number generator
// The argument for the first call must be a NEGATIVE number
double RandomNumberGenerator::rnd(long *randum){
  int i;
  long j;
  static long randum2 = 123456789;
  static long m = 0;
  static long n[ZTAB];
  double tmp;

  if(*randum <= 0){
    if(-(*randum) < 1){
      *randum = 1;
    } else {
      *randum = -(*randum);
    }
    randum2 = (*randum);
    for(i = ZTAB + 7; i >= 0; i--){
      j = (*randum) / RQ1;
      *randum = RA1 * (*randum - j * RQ1) - j * RR1;
      if(*randum < 0){
        *randum += RM1;
      }
      if(i < ZTAB){
        n[i] = *randum;
      }
    }
    m = n[0];
  }
  j = (*randum) / RQ1;
  *randum = RA1 * (*randum - j * RQ1) - j * RR1;
  if(*randum < 0){
    *randum += RM1;
  }
  j = randum2 / RQ2;
  randum2 = RA2 * (randum2 - j * RQ2) - j * RR2;
  if(randum2 < 0){
    randum2 += RM2;
  }
  i = m / ZDIV;
  m = n[i] - randum2;
  n[i] = *randum;
  if(m < 1){
    m += RMM1;
  }
  if((tmp = ZM * m) > ZNMX){
    return ZNMX;
  } else {
    return tmp;
  }
}
