/***************************************************************************
                          randomnumbergenerator.h  -  description
 ***************************************************************************/

/*  Description:
This class implements a random number generator. It is possible to draw values
from different important distributions with different parameters.
*/

#ifndef RANDOMNUMBERGENERATOR_H
#define RANDOMNUMBERGENERATOR_H

#define RM1 2147483563
#define RM2 2147483399
#define ZM (1.0/RM1)
#define RMM1 (RM1-1)
#define RA1 40014
#define RA2 40692
#define RQ1 53668
#define RQ2 52774
#define RR1 12211
#define RR2 3791
#define ZTAB 32
#define ZDIV (1+RMM1/ZTAB)
#define ZPS 1.2e-7
#define ZNMX (1.0-ZPS)
#define PI 3.141592654

#include <math.h>
#include <time.h>
#include "mathfunctions.h"

class RandomNumberGenerator {

public:

  // Constructors and destructors
  RandomNumberGenerator();
  ~RandomNumberGenerator();

  // getSeed: Returns the current seed-value
  long getSeed();

  // init: Initialisation of the random number generator with the current system time
  void init();

  // initWithModifier: Initialisation of the random number generator with the current system time plus an offset
  // (Important for parallel jobs)
  void initWithModifier(double);

  // init: Initialisation of the random counter with an explicit value (it must be NEGATIVE!)
  void init(long);

  // getUniformDistributedNumber: Draws a random number from a uniform probability distribution.
  double getUniformDistributedNumber(double,double);

  // getExtremeValueDistributedNumber: Draws a random number from an extreme value probability distribution.
  double getExtremeValueDistributedNumber(double,double);

  // getNormalDistributedNumber: Draws a random number from a normal probability distribution.
  double getNormalDistributedNumber(double,double);

  // getPoissonDistributedNumber: Draws a random number from a poisson probability distribution.
  double getPoissonDistributedNumber(double);

  // rnd: Random number generator
  double rnd(long*);

  private:
  long seed; // Variable of the random number generator which is used with the rnd-method to generate the next random number. It can be initialised by the system time
};

#endif
