/***************************************************************************
 *   Copyright (C) 2007 by Michael Rosskopf                                *
 *   rosskopf@cs.uni-duesseldorf.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "thetaallelesestimator.h"

// Constructors and destructors
ThetaAllelesEstimator::ThetaAllelesEstimator(){
}
ThetaAllelesEstimator::~ThetaAllelesEstimator(){
}

// calcNumberOfAlleles:
// Calculates the expected number of alleles for a given theta
double ThetaAllelesEstimator::calcNumberOfAlleles(long numberOfIndividuals,double theta){
  double numberOfAlleles = 0;
  for(long i = 0; i < numberOfIndividuals; i++){
    numberOfAlleles = numberOfAlleles + (theta / (theta + i));
  }
  return numberOfAlleles;
}

// estimateThetaFromNumberOfAlleles:
// Estimates theta for a number of alleles observed from data.
double ThetaAllelesEstimator::estimateThetaFromNumberOfAlleles(long numberOfIndividuals,double numberOfAlleles){

  double lowerEstimateForTheta = 0.000001;
  double upperEstimateForTheta = 1.0;
  double newEstimateForTheta;

  // Calculate the maximal value (integer) for theta depending on the number of taxas
  // Note: Greater values would cause nan-errors
  double maximalTheta = 100.0;
  double maximalNumberOfAlleles = 0.0;

  // Find bracketing (limited by the maximal value for theta)
  while((calcNumberOfAlleles(numberOfIndividuals,upperEstimateForTheta) < numberOfAlleles) && (upperEstimateForTheta < maximalTheta)){
    upperEstimateForTheta = upperEstimateForTheta * 2.0;
    if(upperEstimateForTheta > maximalTheta) upperEstimateForTheta = maximalTheta;
  }

  // Find root
  while((upperEstimateForTheta - lowerEstimateForTheta) > 0.00000001){
    newEstimateForTheta = lowerEstimateForTheta + ((upperEstimateForTheta - lowerEstimateForTheta) / 2.0);
    if((calcNumberOfAlleles(numberOfIndividuals,newEstimateForTheta) - numberOfAlleles) < 0){
      lowerEstimateForTheta = newEstimateForTheta;
    }
    if((calcNumberOfAlleles(numberOfIndividuals,newEstimateForTheta) - numberOfAlleles) > 0){
      upperEstimateForTheta = newEstimateForTheta;
    }
  }

  if(numberOfIndividuals == numberOfAlleles) cout << "Warning! No solution if n == a!" << endl;
  if(newEstimateForTheta == 0.0) cout << "Warnung! Extreme value at the lower bound!" << endl;
  if(newEstimateForTheta >= maximalTheta - 0.00000001) cout << "Warnung! Extreme value at the upper bound!" << endl;

  return newEstimateForTheta;
}
